# Generated by Django 4.1.3 on 2022-11-30 00:34

from django.db import migrations, models
from django.conf import settings
import functools
import karrio.server.core.fields
import karrio.server.core.models


def forwards_func(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    Webhook = apps.get_model("events", "Webhook")
    webhooks = (
        Webhook.objects.using(db_alias).raw(
            "SELECT id, to_jsonb(enabled_events) AS enabled_events FROM webhook"
        )
        if "postgres" in settings.DB_ENGINE
        else Webhook.objects.using(db_alias).all()
    )
    _webhooks = []

    for webhook in webhooks:
        webhook.events = webhook.enabled_events
        _webhooks.append(webhook)

    Webhook.objects.using(db_alias).bulk_update(_webhooks, ["events"])


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        ("events", "0005_event_event_object_idx"),
    ]

    operations = [
        migrations.AddField(
            model_name="webhook",
            name="events",
            field=karrio.server.core.fields.MultiChoiceField(
                choices=[
                    ("all", "all"),
                    ("shipment_purchased", "shipment_purchased"),
                    ("shipment_cancelled", "shipment_cancelled"),
                    ("shipment_fulfilled", "shipment_fulfilled"),
                    ("tracker_created", "tracker_created"),
                    ("tracker_updated", "tracker_updated"),
                    ("order_created", "order_created"),
                    ("order_updated", "order_updated"),
                    ("order_fulfilled", "order_fulfilled"),
                    ("order_cancelled", "order_cancelled"),
                    ("order_delivered", "order_delivered"),
                    ("batch_queued", "batch_queued"),
                    ("batch_failed", "batch_failed"),
                    ("batch_running", "batch_running"),
                    ("batch_completed", "batch_completed"),
                ],
                default=functools.partial(
                    karrio.server.core.models._identity, *(), **{"value": []}
                ),
                help_text="Webhook events",
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="data",
            field=models.JSONField(
                default=functools.partial(
                    karrio.server.core.models._identity, *(), **{"value": {}}
                )
            ),
        ),
        migrations.RunPython(forwards_func, reverse_func),
        migrations.RemoveField(
            model_name="webhook",
            name="enabled_events",
        ),
        migrations.RenameField(
            model_name="webhook",
            old_name="events",
            new_name="enabled_events",
        ),
    ]
